S3method(print,lprobust)
S3method(summary,lprobust)
S3method(print,summary.lprobust)
S3method(plot,lprobust)
S3method(print,lpbwselect)
S3method(summary,lpbwselect)
S3method(print,summary.lpbwselect)
S3method(print,kdrobust)
S3method(summary,kdrobust)
S3method(print,summary.kdrobust)
S3method(plot,kdrobust)
S3method(print,kdbwselect)
S3method(summary,kdbwselect)
S3method(print,summary.kdbwselect)
S3method(ggplot2::autoplot,lprobust)
S3method(ggplot2::autoplot,kdrobust)
S3method(broom::tidy,lprobust)
S3method(broom::glance,lprobust)
S3method(broom::tidy,kdrobust)
S3method(broom::glance,kdrobust)
S3method(broom::tidy,lpbwselect)
S3method(broom::glance,lpbwselect)
S3method(broom::tidy,kdbwselect)
S3method(broom::glance,kdbwselect)
export(lpbwselect)
export(lprobust)
export(kdbwselect)
export(kdrobust)
export(nprobust.plot)
import(ggplot2)
importFrom("stats","integrate","complete.cases","deriv","dnorm","kernel","lm","optimize","pnorm","poly","qnorm","quantile","sd","sigma")
importFrom("graphics","lines","plot","polygon")

